
import java.lang.*;
import java.io.*;

public class DacapoLauncher {

	private static final int WARMUPS = 5;

	private static final int ITERATIONS = WARMUPS + 1;
	private static final String[] COMMAND_TEMPLATE = ("-jar dacapo-9.12-bach.jar --iterations " + ITERATIONS + " --size").split(" ");
	private static final String RESULT_LINE_HEADER = "===== DaCapo 9.12", RESULT_INTRO = "PASSED in ";
	private static final String ABORT_INDICATOR = "FAILED";
	private static final int RETRIES = 2;

	private static final int TIMEOUT = -1, CRASHED = -2;

	public static void main(String[] args) {
		String instanceName = args[0].substring(args[0].lastIndexOf('/') + 1);
		String instanceSpecificInformation = args[1];
		double cutoffTime = Double.parseDouble(args[2]);
		String cutoffLength = args[3];
		String seed = args[4];
		String[] params = new String[(args.length - 5) / 2];
		for(int i = 0; i < params.length; i++) {
			String name = args[5 + (i*2) + 0];
			String value = args[5 + (i*2) + 1];
			if(value.startsWith("'") && value.endsWith("'")) {
				value = value.substring(1, value.length() - 1);
			}
			String param;
			if(name.startsWith("-XX:?")) {
				param = name.replace('?', value.equals("0") ? '-' : '+');
			} else {
				param = name + "=" + value;
			}
			params[i] = param;
		}

		String[] command = new String[1 + params.length + COMMAND_TEMPLATE.length + 2];
		command[0] = System.getProperties().getProperty("java.home") + "/bin/java";
		for(int i = 0; i < params.length; i++) {
			command[1 + i] = params[i];
		}
		for(int i = 0; i < COMMAND_TEMPLATE.length; i++) {
			command[1 + params.length + i] = COMMAND_TEMPLATE[i];
		}
		command[command.length - 2] = instanceSpecificInformation;
		command[command.length - 1] = instanceName;

		try {
			int result = 0;
			for(int i = 0; result == 0 && i < RETRIES + 1; i++) {
				result = execute(command, (int) (cutoffTime * 1000 * ITERATIONS * 2)); //assumes that first run takes twice as long at most!
			}
			String resultType;
			switch(result) {
				case CRASHED: resultType = "CRASHED"; break;
				case TIMEOUT: resultType = "TIMEOUT"; break;
				default: resultType = "SAT"; break;
			}
			System.out.println("Result for ParamILS: " + resultType + ", " + (result > 0 ? result / 1000.0 : 0) + ", 0, 0, " + seed);
		} catch(Throwable t) {
			t.printStackTrace();
		}
	}

	private final static class ProcessInfo {
		public boolean aborted = false;
	}

	private static int execute(String[] command, final int timeout) throws Throwable {
		final Object monitor = new Object();
		final ProcessInfo info = new ProcessInfo();
		final ProcessBuilder builder = new ProcessBuilder();
		builder.command(command);
		final Process process = builder.start();
		
		final Thread killer = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					Thread.sleep(timeout);
				} catch (InterruptedException ie) {
				} finally {
					synchronized(monitor) {
						process.destroy();
						info.aborted = true;
					}
				}
			}
		});
		killer.start();

		int result = 0;

		try(BufferedReader reader = new BufferedReader(new InputStreamReader(process.getErrorStream()))) {
			for(String line = reader.readLine(); line != null; line = reader.readLine()) {
				if(line.startsWith(RESULT_LINE_HEADER) && line.contains(RESULT_INTRO)) {
					int start = line.indexOf(RESULT_INTRO) + RESULT_INTRO.length();
					result = Integer.parseInt(line.substring(start, line.indexOf(' ', start + 1)));
					assert result > 0;
					break;
				} else if (line.contains(ABORT_INDICATOR)) {
					break;
				}
			}
		} finally {
			synchronized(monitor) {
				process.waitFor();
				if(info.aborted) {
					result = TIMEOUT;
				} else {
					if(result == 0 && process.exitValue() != 0) {
						result = CRASHED;
					}
					killer.interrupt();
				}
			}
		}

		return result;
	}

}

