# Copyright (c) 2014, 2015, dynatrace and/or its affiliates. All rights reserved.
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 only, as
# published by the Free Software Foundation.
#
# This code is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# version 2 for more details (a copy is included in the LICENSE file that
# accompanied this code).
# 
# You should have received a copy of the GNU General Public License
# along with with this work.  If not, see <http://www.gnu.org/licenses/>.
#

name="$1"
timeout="$2"
executable="$3"
fixed_params=$(echo "$4" | sed -r 's_\S+_\"&\",_g') # Introduces "" around words and inserts comma
params_file="$5"
command=$(echo "$6" | sed -r 's_\S+_\"&\",_g') # Introduces "" around words and inserts comma

date="$(date +%Y_%m_%d_%H_%M_%S)"
out_dir="results/result_""$name""_""$instance""_""$date"
log_dir="$out_dir/.logs"

result_file="$out_dir/result.txt"

paramils_log="$out_dir/paramils.log"
paramils_stats="$out_dir/paramils.stats"


rm ./.abort 2> /dev/null

# Starting the abortion subshell
(
sleep $timeout # Sleep x seconds
echo "abort" > ./.abort
) &

# Main shell
mkdir -p "$out_dir" &&
mkdir -p "$log_dir" &&
sed -e 's?###EXECUTABLE###?'"$executable"'?' -e 's?###FIXED_PARAMS###?'"$fixed_params"'?' -e 's?###COMMAND###?'"$command"'?' < Launcher.java.template > Launcher.java &&
javac Launcher.java &&
paramils2.3.5-source/bin/paramils -numRun 0 -execdir "." -algo "java -ea -cp . Launcher 0 $timeout" -deterministic 1 -run_obj runtime -overall_obj mean -cutoff_time max -cutoff_length max -tunerTimeout "$timeout" -maxEvals "$timeout" -maxIts "$timeout" -paramfile "$params_file" -outdir "$log_dir" -instance_file "instance.txt" -test_instance_file "instance.txt" | tee "$paramils_log" | java -jar VMConfigurationParser.jar ParamILS "$executable" > "$result_file" &&
touch "$paramils_stats" &&
echo "runs_succeeded = $(cat $paramils_log | grep SAT, | wc -l)"	>> "$paramils_stats" &&
echo "runs_timedout = $(cat $paramils_log | grep TIMEOUT, | wc -l)"	>> "$paramils_stats" &&
echo "runs_crashed = $(cat $paramils_log | grep CRASHED, | wc -l)"	>> "$paramils_stats" &&
echo "runs_unknown = $(cat $paramils_log | grep UNSAT, | wc -l)"	>> "$paramils_stats"

kill %1

rm .abort 2> /dev/null
